import { Children, isValidElement, cloneElement, forwardRef, useRef } from "react";
import { useForkRef, useForkHandler } from "../hooks";
import { Box } from "./box";
import { Floater } from "./floater";
import { ClickAwayListener } from "./click-away-listener";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
export const Dropdown = /*#__PURE__*/forwardRef((_ref, ref) => {
  let {
    overlay,
    placement = "bottom-start",
    isOpen,
    onOpen,
    onClose,
    disabled,
    keepMounted = true,
    matchWidth = true,
    color,
    children,
    ...rest
  } = _ref;
  const handleClick = () => {
    if (!disabled) {
      if (isOpen) {
        onClose?.();
      } else {
        onOpen?.();
      }
    }
  };
  const handleClickAway = () => {
    if (isOpen) {
      onClose?.();
    }
  };
  const child = Children.only(children);
  const anchorRef = useRef();
  const childProps = {
    ref: useForkRef(child.ref, anchorRef),
    onClick: useForkHandler(child.props.onClick, handleClick)
  };
  const anchor = /*#__PURE__*/isValidElement(child) ? /*#__PURE__*/cloneElement(child, childProps) : child;
  return /*#__PURE__*/_jsx(ClickAwayListener, {
    onClickAway: handleClickAway,
    children: /*#__PURE__*/_jsxs("div", {
      children: [anchor, /*#__PURE__*/_jsx(Floater, {
        ref: ref,
        offset: 4,
        anchor: anchorRef.current,
        isOpen: isOpen,
        placement: placement,
        keepMounted: keepMounted,
        matchWidth: matchWidth,
        onMouseDown: e => e.preventDefault(),
        children: /*#__PURE__*/_jsx(Box, {
          py: 1,
          borderRadius: "base",
          boxShadow: "base",
          bg: "dropdown.bg",
          color: color,
          ...rest,
          children: overlay
        })
      })]
    })
  });
});
if (process.env.NODE_ENV !== "production") {
  Dropdown.displayName = "Dropdown";
}