import { forwardRef, createContext, useContext, useRef, useEffect } from "react";
import { RemoveScroll } from "react-remove-scroll";
// @ts-ignore
import { motion, AnimatePresence } from "framer-motion/dist/framer-motion";
import { suppressOthers } from "aria-hidden";
import styled from "styled-components";
import { useCallbackRef, useForkHandler } from "../hooks";
import { CloseIcon } from "../icons";
import { KeyCode } from "../utils/key-code";
import { Flex } from "./flex";
import { Box } from "./box";
import { Portal } from "./portal";
import { FocusLock } from "./focus-lock";
import { IconButton } from "./icon-button";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
const ModalContext = /*#__PURE__*/createContext({});
const OverlayWrap = styled(Box).withConfig({
  displayName: "modal__OverlayWrap",
  componentId: "sc-d83j7x-0"
})(["position:fixed;top:0;left:0;width:100vw;height:100vh;z-index:1;"]);
const ContentWrap = styled(Flex).withConfig({
  displayName: "modal__ContentWrap",
  componentId: "sc-d83j7x-1"
})(["position:fixed;top:0;left:0;width:100vw;height:100vh;z-index:1;"]);
const ContentBox = motion(Flex);
const Overlay = /*#__PURE__*/forwardRef((_ref, ref) => {
  let {
    color,
    ...rest
  } = _ref;
  return /*#__PURE__*/_jsx(OverlayWrap, {
    ref: ref,
    bg: "alpha.black.6",
    ...rest,
    color: color
  });
});
const Content = /*#__PURE__*/forwardRef((props, ref) => {
  const {
    onClose,
    onKeyDown,
    onClick,
    color,
    children,
    ...rest
  } = props;
  const {
    scrollBehavior,
    autoFocus,
    restoreFocus,
    initialFocusRef,
    finalFocusRef
  } = useContext(ModalContext);
  const contentRef = useRef(null);
  const mouseDownTarget = useRef(null);
  useEffect(() => {
    if (contentRef.current) {
      return suppressOthers(contentRef.current);
    }
    return;
  }, []);
  const onMouseDown = event => {
    mouseDownTarget.current = event.target;
  };
  const onClickAway = event => {
    if (mouseDownTarget.current === event.target) {
      onClose?.(event);
    }
  };
  const onActivation = useCallbackRef(() => {
    initialFocusRef?.current?.focus();
  });
  const onDeactivation = useCallbackRef(() => {
    finalFocusRef?.current?.focus();
  });
  const sectionProps = {
    onClick: useForkHandler(onClick, e => e.stopPropagation()),
    ...rest
  };
  return /*#__PURE__*/_jsx(FocusLock, {
    autoFocus: autoFocus,
    returnFocus: restoreFocus && !finalFocusRef,
    onActivation: onActivation,
    onDeactivation: onDeactivation,
    children: /*#__PURE__*/_jsx(RemoveScroll, {
      children: /*#__PURE__*/_jsx(ContentWrap, {
        ref: contentRef,
        role: "dialog",
        justify: "center",
        align: "flex-start",
        overflow: scrollBehavior === "inside" ? "hidden" : "auto",
        tabIndex: -1,
        "aria-modal": true,
        onKeyDown: onKeyDown,
        onMouseDown: onMouseDown,
        onClick: onClickAway,
        children: /*#__PURE__*/_jsx(ContentBox, {
          ref: ref,
          as: "section",
          direction: "column",
          position: "relative",
          borderRadius: "base",
          boxShadow: "base",
          bg: "modal.bg",
          pointerEvents: "initial",
          my: 5,
          width: 1,
          maxWidth: 448,
          maxHeight: scrollBehavior === "inside" ? "calc(100% - 128px)" : "",
          color: color,
          initial: {
            opacity: 0,
            scale: 0.8
          },
          animate: {
            opacity: 1,
            scale: 1
          },
          exit: {
            opacity: 0,
            scale: 0.8
          },
          transition: {
            type: "spring",
            damping: 20,
            stiffness: 300
          },
          ...sectionProps,
          children: children
        })
      })
    })
  });
});
const Header = /*#__PURE__*/forwardRef((_ref2, ref) => {
  let {
    color = "heading",
    ...rest
  } = _ref2;
  return /*#__PURE__*/_jsx(Box, {
    ref: ref,
    as: "header",
    flex: 0,
    fontSize: 3,
    p: 3,
    color: color,
    ...rest
  });
});
const Body = /*#__PURE__*/forwardRef((_ref3, ref) => {
  let {
    color,
    ...rest
  } = _ref3;
  const {
    scrollBehavior
  } = useContext(ModalContext);
  const overflow = scrollBehavior === "inside" ? "auto" : "";
  return /*#__PURE__*/_jsx(Box, {
    ref: ref,
    flex: 1,
    lineHeight: "base",
    pt: 2,
    pb: 3,
    px: 3,
    overflow: overflow,
    color: color,
    ...rest
  });
});
const Footer = /*#__PURE__*/forwardRef((_ref4, ref) => {
  let {
    color,
    ...rest
  } = _ref4;
  return /*#__PURE__*/_jsx(Flex, {
    ref: ref,
    as: "footer",
    justify: "flex-end",
    align: "center",
    flex: 0,
    gap: 2,
    p: 3,
    color: color,
    ...rest
  });
});
const CloseButton = /*#__PURE__*/forwardRef((props, ref) => {
  return /*#__PURE__*/_jsx(IconButton, {
    ref: ref,
    type: "button",
    variant: "text",
    position: "absolute",
    top: "8px",
    right: "8px",
    "aria-label": "Close",
    icon: /*#__PURE__*/_jsx(CloseIcon, {
      size: 24
    }),
    ...props
  });
});
export function Modal(_ref5) {
  let {
    isOpen,
    onClose,
    scrollBehavior = "outside",
    autoFocus = true,
    restoreFocus = true,
    initialFocusRef,
    finalFocusRef,
    overlayRef,
    contentRef,
    container,
    className,
    children,
    ...rest
  } = _ref5;
  const context = {
    scrollBehavior,
    autoFocus,
    restoreFocus,
    initialFocusRef,
    finalFocusRef
  };
  const handleClose = event => {
    event.stopPropagation();
    onClose?.();
  };
  const handleKeyDown = event => {
    if (event.key === KeyCode.ESC) {
      event.stopPropagation();
      onClose?.();
    }
  };
  return /*#__PURE__*/_jsx(ModalContext.Provider, {
    value: context,
    children: /*#__PURE__*/_jsx(AnimatePresence, {
      children: isOpen ? /*#__PURE__*/_jsxs(Portal, {
        container: container,
        children: [/*#__PURE__*/_jsx(Overlay, {
          ref: overlayRef
        }), /*#__PURE__*/_jsxs(Content, {
          ref: contentRef,
          ...rest,
          className: className,
          onClose: handleClose,
          onKeyDown: handleKeyDown,
          children: [/*#__PURE__*/_jsx(CloseButton, {
            onClick: handleClose
          }), children]
        })]
      }) : null
    })
  });
}
Modal.Header = Header;
Modal.Body = Body;
Modal.Footer = Footer;
if (process.env.NODE_ENV !== "production") {
  Modal.displayName = "Modal";
}