import { Children, isValidElement, cloneElement, forwardRef, useState, useEffect, useLayoutEffect } from "react";
import { useFloating, autoUpdate, offset, flip, shift, arrow, size } from "@floating-ui/react-dom";
// @ts-ignore
import { motion, AnimatePresence } from "framer-motion/dist/framer-motion";
import { color } from "styled-system";
import styled from "styled-components";
import { useDisclosure, useForkRef, useEventListener } from "../hooks";
import { Box } from "./box";
import { Portal } from "./portal";
import { ClickAwayListener } from "./click-away-listener";
import { KeyCode } from "../utils/key-code";
import { getStaticSide, getTransformOrigin } from "../utils/floating-util";
import { isNumber } from "../utils/type-util";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
import { Fragment as _Fragment } from "react/jsx-runtime";
const Arrow = styled.div.withConfig({
  displayName: "popover__Arrow",
  componentId: "sc-m7ihma-0"
})(["position:absolute;width:8px;height:8px;transform:rotate(45deg);overflow:hidden;pointer-events:none;", ""], color);
export const Popover = /*#__PURE__*/forwardRef((_ref, ref) => {
  let {
    title,
    content,
    placement = "top",
    strategy = "absolute",
    trigger = "hover",
    mouseEnterDelay = 100,
    mouseLeaveDelay = 100,
    isOpen: isOpenProp,
    defaultIsOpen,
    onOpen,
    onClose,
    color,
    container,
    children,
    ...rest
  } = _ref;
  const {
    isOpen,
    open,
    close,
    toggle,
    clearDelayTimer
  } = useDisclosure({
    isOpen: isOpenProp,
    defaultIsOpen,
    onOpen,
    onClose
  });
  const openWithDelay = () => open({
    delay: mouseEnterDelay
  });
  const closeWithDelay = () => close({
    delay: mouseLeaveDelay
  });
  const handleMouseEnter = () => {
    if (trigger === "hover") {
      clearDelayTimer();
    }
  };
  const handleMouseLeave = () => {
    if (trigger === "hover") {
      closeWithDelay();
    }
  };
  const handleKeyDown = event => {
    if (trigger === "click" && event.key === KeyCode.ESC && isOpen) {
      close();
    }
  };
  const handleClickAway = event => {
    if (!anchorNode || !event.composedPath().includes(anchorNode)) {
      closeWithDelay();
    }
  };
  const [anchorNode, setAnchorNode] = useState(null);
  const [arrowNode, setArrowNode] = useState(null);
  const child = Children.only(children);
  const childProps = {
    ref: useForkRef(child.ref, setAnchorNode)
  };
  const anchor = /*#__PURE__*/isValidElement(child) ? /*#__PURE__*/cloneElement(child, childProps) : child;
  const middleware = [offset(12), flip({
    padding: 12
  }), shift({
    padding: 12
  }), size()];
  if (arrowNode) {
    middleware.push(arrow({
      element: arrowNode,
      padding: 8
    }));
  }
  const {
    x,
    y,
    refs,
    reference,
    floating,
    update,
    middlewareData,
    placement: autoPlacement
  } = useFloating({
    strategy,
    placement,
    middleware
  });
  useLayoutEffect(() => reference(anchorNode), [anchorNode]);
  useLayoutEffect(() => update(), [isOpen, refs.floating.current]);
  useLayoutEffect(() => {
    if (arrowNode) {
      const {
        x,
        y
      } = middlewareData.arrow ?? {};
      const staticSide = getStaticSide(autoPlacement);
      Object.assign(arrowNode.style, {
        left: isNumber(x) ? `${x}px` : "",
        top: isNumber(y) ? `${y}px` : "",
        right: "",
        bottom: "",
        [staticSide]: "-3.5px"
      });
    }
  });
  useEffect(() => {
    if (isOpen && refs.reference.current && refs.floating.current) {
      return autoUpdate(refs.reference.current, refs.floating.current, update);
    }
    return;
  }, [isOpen, refs.reference.current, refs.floating.current, update]);
  useEventListener("mouseenter", openWithDelay, trigger === "hover" ? anchorNode : null);
  useEventListener("mouseleave", closeWithDelay, trigger === "hover" ? anchorNode : null);
  useEventListener("click", () => toggle(), trigger === "click" ? anchorNode : null);
  useEventListener("keydown", handleKeyDown, trigger === "click" ? anchorNode : null);
  return /*#__PURE__*/_jsxs(_Fragment, {
    children: [anchor, /*#__PURE__*/_jsx(AnimatePresence, {
      children: isOpen ? /*#__PURE__*/_jsx(Portal, {
        container: container,
        children: /*#__PURE__*/_jsx(motion.div, {
          ref: floating,
          role: "tooltip",
          initial: {
            opacity: 0,
            scale: 0.8
          },
          animate: {
            opacity: 1,
            scale: 1
          },
          exit: {
            opacity: 0,
            scale: 0.8
          },
          transition: {
            type: "spring",
            damping: 20,
            stiffness: 300
          },
          style: {
            position: strategy,
            top: y ?? "",
            left: x ?? "",
            transformOrigin: getTransformOrigin(autoPlacement),
            zIndex: 1
          },
          children: /*#__PURE__*/_jsx(ClickAwayListener, {
            mouseEvent: "onMouseDown",
            onClickAway: handleClickAway,
            children: /*#__PURE__*/_jsxs(Box, {
              ref: ref,
              minWidth: 120,
              maxWidth: 300,
              lineHeight: "base",
              borderRadius: "base",
              boxShadow: "base",
              bg: "popover.bg",
              userSelect: "text",
              onMouseEnter: handleMouseEnter,
              onMouseLeave: handleMouseLeave,
              color: color,
              ...rest,
              children: [/*#__PURE__*/_jsx(Arrow, {
                ref: setArrowNode,
                bg: "popover.bg"
              }), title ? /*#__PURE__*/_jsx(Box, {
                p: 2,
                borderBottom: "split",
                color: "heading",
                fontWeight: "bold",
                children: title
              }) : null, /*#__PURE__*/_jsx(Box, {
                p: 2,
                children: content
              })]
            })
          })
        })
      }) : null
    })]
  });
});
if (process.env.NODE_ENV !== "production") {
  Popover.displayName = "Popover";
}