import { forwardRef } from "react";
import styled, { css } from "styled-components";
import { useDisclosure, useControllableState, useForkHandler } from "../hooks";
import { ChevronDownIcon, CloseIcon } from "../icons";
import { KeyCode } from "../utils/key-code";
import { Input } from "./input";
import { Dropdown } from "./dropdown";
import { ScrollBox } from "./scroll-box";
import { Flex } from "./flex";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
const Option = styled.div.withConfig({
  displayName: "select__Option",
  componentId: "sc-1666d02-0"
})(["padding:4px;flex:auto;overflow:hidden;white-space:nowrap;text-overflow:ellipsis;cursor:pointer;&:hover,&:active{color:", ";background:", ";}", ""], p => p.theme.colors.gray[9], p => p.theme.colors.alpha[1], p => p.selected && css(["&,&:hover{color:", ";background:", ";}"], p.theme.colors.primary.base, p.theme.colors.gray[1]));
const SelectArrow = styled.span.withConfig({
  displayName: "select__SelectArrow",
  componentId: "sc-1666d02-1"
})(["display:contents;"]);
const SelectClear = styled.span.withConfig({
  displayName: "select__SelectClear",
  componentId: "sc-1666d02-2"
})(["color:", ";display:none;"], p => p.theme.colors.text);
const StyledSuffix = styled(Input.Suffix).withConfig({
  displayName: "select__StyledSuffix",
  componentId: "sc-1666d02-3"
})(["pointer-events:none;"]);
const StyledGroup = styled(Input.Group).withConfig({
  displayName: "select__StyledGroup",
  componentId: "sc-1666d02-4"
})(["&[data-allow-clear=\"true\"]{input:enabled ~ ", "{pointer-events:auto;cursor:pointer;}&:hover{input:enabled ~ ", "{", "{display:contents;}", "{display:none;}}}}"], StyledSuffix, StyledSuffix, SelectClear, SelectArrow);
export const Select = /*#__PURE__*/forwardRef((_ref, ref) => {
  let {
    options = [],
    value,
    defaultValue,
    onChange,
    onOpen,
    onClose,
    onBlur,
    onKeyDown,
    disabled,
    suffixIcon,
    clearIcon,
    allowClear = false,
    maxScrollHeight = 256,
    renderOverlay,
    className,
    ...rest
  } = _ref;
  const {
    isOpen,
    open,
    close
  } = useDisclosure({
    onOpen,
    onClose
  });
  const [selectedValue, setSelectedValue] = useControllableState(value, defaultValue);
  const selectedOption = options.find(opt => opt.value === selectedValue);
  const handleBlur = () => {
    close();
  };
  const handleKeyDown = event => {
    if (event.key === KeyCode.ENTER) {
      open();
    } else if (event.key === KeyCode.ESC) {
      event.stopPropagation();
      close();
    }
  };
  const handleSelect = val => {
    setSelectedValue(val);
    onChange?.(val);
    close();
  };
  const handleClear = event => {
    if (allowClear) {
      event.preventDefault();
      handleSelect("");
    }
  };
  const inputProps = {
    onBlur: useForkHandler(onBlur, handleBlur),
    onKeyDown: useForkHandler(onKeyDown, handleKeyDown),
    disabled,
    ...rest
  };
  const overlayItems = /*#__PURE__*/_jsx(ScrollBox, {
    maxHeight: maxScrollHeight,
    children: options.length > 0 ? options.map(opt => /*#__PURE__*/_jsx(Option, {
      onClick: () => handleSelect(opt.value),
      selected: selectedOption?.value === opt.value,
      children: opt.children ?? opt.label
    }, opt.value)) : /*#__PURE__*/_jsx(Flex, {
      p: 1,
      align: "center",
      justify: "center",
      color: "placeholder",
      children: "No Data"
    })
  });
  const overlay = renderOverlay?.(overlayItems) ?? overlayItems;
  return /*#__PURE__*/_jsx(Dropdown, {
    overlay: overlay,
    isOpen: isOpen,
    onOpen: open,
    onClose: close,
    disabled: disabled,
    children: /*#__PURE__*/_jsxs(StyledGroup, {
      className: className,
      "data-allow-clear": allowClear && !!selectedValue,
      children: [/*#__PURE__*/_jsx(Input, {
        ref: ref,
        value: selectedOption?.label ?? selectedValue ?? "",
        role: "combobox",
        cursor: "pointer",
        readOnly: true,
        ...inputProps
      }), /*#__PURE__*/_jsxs(StyledSuffix, {
        onClick: handleClear,
        "aria-hidden": true,
        children: [/*#__PURE__*/_jsx(SelectArrow, {
          children: suffixIcon ?? /*#__PURE__*/_jsx(ChevronDownIcon, {})
        }), /*#__PURE__*/_jsx(SelectClear, {
          children: clearIcon ?? /*#__PURE__*/_jsx(CloseIcon, {})
        })]
      })]
    })
  });
});
if (process.env.NODE_ENV !== "production") {
  Select.displayName = "Select";
}