import { forwardRef, useState } from "react";
import styled from "styled-components";
import { useControllableState } from "../hooks";
import { Label } from "./label";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
function getCellColor(_ref) {
  let {
    theme,
    active,
    selectionActive
  } = _ref;
  if (selectionActive) {
    return theme.colors.primary[2];
  }
  if (active) {
    return theme.colors.primary.base;
  }
  return theme.colors.alpha[1];
}
const StyledCell = styled.td.withConfig({
  displayName: "size-picker__StyledCell",
  componentId: "sc-lsy18u-0"
})(["&::before{background:", ";border-radius:", "px;height:100%;display:block;content:\"\";}"], getCellColor, p => p.theme.radii.base);
const StyledTable = styled.table.withConfig({
  displayName: "size-picker__StyledTable",
  componentId: "sc-lsy18u-1"
})(["border:0;border-spacing:0;border-collapse:collapse;tbody{td{width:32px;height:32px;padding:4px;cursor:pointer;-webkit-touch-callout:none;user-select:none;}}"]);
function DataCell(_ref2) {
  let {
    width,
    height,
    value,
    selection,
    onSelectionChange
  } = _ref2;
  const active = !selection && width <= value[0] && height <= value[1];
  const selectionActive = !!selection && width <= selection.width && height <= selection.height;
  const onMouseMove = event => {
    event.preventDefault();
    if (!selection || selection.width !== width || selection.height !== height) {
      onSelectionChange({
        width,
        height
      });
    }
  };
  return /*#__PURE__*/_jsx(StyledCell, {
    active: active,
    selectionActive: selectionActive,
    onMouseMove: onMouseMove
  });
}
export const SizePicker = /*#__PURE__*/forwardRef((_ref3, ref) => {
  let {
    value,
    defaultValue,
    max,
    onChange
  } = _ref3;
  const defaultSize = [1, 1];
  const [selection, setSelection] = useState(null);
  const [size, setSize] = useControllableState(value, defaultValue ?? defaultSize);
  const [width, height] = size;
  const [maxWidth, maxHeight] = max ?? defaultSize;
  const rows = [...Array(maxHeight).keys()];
  const columns = [...Array(maxWidth).keys()];
  const onClick = () => {
    if (selection) {
      const nextValue = [selection.width, selection.height];
      setSize(nextValue);
      onChange?.(nextValue);
    }
  };
  const onMouseLeave = () => {
    if (selection) {
      setSelection(null);
    }
  };
  return /*#__PURE__*/_jsxs("div", {
    ref: ref,
    children: [/*#__PURE__*/_jsx(StyledTable, {
      onClick: onClick,
      onMouseLeave: onMouseLeave,
      children: /*#__PURE__*/_jsx("tbody", {
        children: rows.map(row => /*#__PURE__*/_jsx("tr", {
          children: columns.map(col => /*#__PURE__*/_jsx(DataCell, {
            width: col + 1,
            height: row + 1,
            value: size,
            selection: selection,
            onSelectionChange: setSelection
          }, col))
        }, row))
      })
    }), /*#__PURE__*/_jsxs(Label, {
      p: 1,
      children: [selection?.width ?? width, "x", selection?.height ?? height]
    })]
  });
});
if (process.env.NODE_ENV !== "production") {
  SizePicker.displayName = "SizePicker";
}